/* This program draws a filled yin-yang symbol, and some stuffs that moves around in harmonic motions. */

void setup()
{ 
  size(900, 900);    
  noFill();
  smooth();
  background(222, 255);
}

void draw()
{
  float maxInt = 2147483647; // 2^31-1
  float pi = 3.1415927;  
  
  float middleX = 450; // x-coordinate of the big circle's center
  float middleY = 450; // y-coordinate of the big circle's center
  float radiusBig = 300; // the radius of the big circle
  float rangeY = 100; // the absolute magnitude of the sin wave
  int frequency = 20000000; // increase to make things move faster  
  
  float t1 = (frameCount * frequency / maxInt); // map frameCount to range [-1, 1]    
  t1 = t1 * pi; // map t1 to range [-pi, pi]    
  // Finite Taylor expansion for sin(t1) (works OK for t1 in [-pi, pi] only)
  float sin_t1 = t1 - t1*t1*t1/6 + t1*t1*t1*t1*t1/120 - t1*t1*t1*t1*t1*t1*t1/5040;     
  float sinPointTopX = middleX + t1 / pi * radiusBig;
  float sinPointTopY = middleY + sin_t1 * rangeY;  
         
  /* we fill each half of the big circle by repeatedly drawing a line between to moving points.
     the first point runs from left to right near the edge of the big circle.
     the second point runs from left to right in a sin wave near the middle of the circle. */
  float edgePointTopX =  middleX + t1 / pi * radiusBig;    
  float s = radiusBig * radiusBig - (middleX - edgePointTopX) * (middleX - edgePointTopX); // s = radius^2 - x^2    
  /* the following five lines try to estimate the edge point's y coordinate, given the x coordinate.
     this is simply y = sqrt(radius^2 - x^2), but since the sqrt() function has not been covered,
     we must resort to some trick... */
  float ry = (radiusBig / 2 + s / (radiusBig / 2)) / 2;
  ry = (ry + s / ry) / 2; ry = (ry + s / ry) / 2;
  ry = (ry + s / ry) / 2; ry = (ry + s / ry) / 2;
  float edgePointTopY = middleY - ry;
  float edgePointBottomX = middleX * 2 - edgePointTopX;
  float edgePointBottomY = middleY * 2 - edgePointTopY;    
  float sinPointBottomX = middleX * 2 - sinPointTopX;
  float sinPointBottomY = middleY * 2 - sinPointTopY;  
  
  /* actually drawing the lines */
  strokeWeight(20);  
  /* top half */  
  stroke(255, 156, 25, 20);
  line(edgePointTopX, edgePointTopY, sinPointTopX, sinPointTopY);  
  fill(255, 255, 255, 20); noStroke();
  ellipse(edgePointTopX, edgePointTopY, 20, 20);
  /* bottom half */
  stroke(0, 147, 229, 20); noFill();
  line(edgePointBottomX, edgePointBottomY, sinPointBottomX, sinPointBottomY);
  fill(255, 255, 255, 20); noStroke();
  ellipse(middleX * 2 - edgePointTopX, middleY * 2 - edgePointTopY, 20, 20);
    
  /* next, we draw two small circles and the things that revolve around them */
  // the Taylor expansion for cosine...
  float cos_t1 = 1 - t1*t1 / 2 + t1*t1*t1*t1 / 24 - t1*t1*t1*t1*t1*t1/720;
  float tan_t1 = sin_t1 / cos_t1; // unnoticeable discontinuous near cos_t1 = 0
  /* the following two lines use tangent formulas for half angles to double the frequency */
  float cos_2_t1 = (1 - tan_t1 * tan_t1) / (1 + tan_t1 * tan_t1);
  float sin_2_t1 = (2 * tan_t1) / (1 + tan_t1 * tan_t1);
  float offsetX = 100; // x offset from the center x
  float offsetY = 150; // y offset from the center y
  float radiusSmall = 20;  
  noStroke();  
  /* top one */
  fill(0, 147, 229, 10);
  noStroke();
  ellipse(middleX + offsetX, middleY - offsetY, radiusSmall * 2, radiusSmall * 2);            
  /* bottom one */
  fill(255, 156, 25, 10);
  noStroke();
  ellipse(middleX - offsetX, middleY + offsetY, radiusSmall * 2, radiusSmall * 2);  
  /* "circulating" white borders */
  fill(222, 222, 222, 20);  
  ellipse(middleX + offsetX + radiusSmall * cos_2_t1, middleY - offsetY + radiusSmall * sin_2_t1, 16, 16);                                   
  ellipse(middleX - offsetX - radiusSmall * cos_2_t1, middleY + offsetY + radiusSmall * sin_2_t1, 16, 16);        
  
  /* draw some squares at integer coordinates for a "rasterizing" effect */  
  noFill(); strokeWeight(2); stroke(200, 215, 179, 5);
  float squareSize = 90;
  rect(int(edgePointTopX/squareSize)*squareSize, int(edgePointTopY/squareSize)*squareSize, squareSize, squareSize);    
  rect(int(edgePointBottomX/squareSize)*squareSize, int(edgePointBottomY/squareSize)*squareSize, squareSize, squareSize);  
  rect(int(sinPointTopX/squareSize)*squareSize, int(sinPointTopY/squareSize)*squareSize, squareSize, squareSize);  
  
  /* here we draw the white "shuttles" that moves back and forth in the sin wave */
  float t2 = ((frameCount + 5) * frequency / maxInt);
  t2 = t2 * pi;  
  float sin_t2 = t2 - t2*t2*t2/6 + t2*t2*t2*t2*t2/120 - t2*t2*t2*t2*t2*t2*t2/5040;
  /* left-to-right */
  float leftRightX = middleX + t2 / pi * radiusBig;
  float leftRightY = middleY + sin_t2 * rangeY;  
  fill(255, 255, 255, 50); ellipse(leftRightX, leftRightY, 25, 25);
  fill(23, 233, 76, 50); ellipse(sinPointTopX, sinPointTopY, 20, 20);  
  /* right-to-left */
  float rightLeftX = middleX - t2 / pi * radiusBig;
  float rightLeftY = middleY - sin_t2 * rangeY;
  fill(255, 255, 255, 50); ellipse(rightLeftX, rightLeftY, 25, 25);
  fill(23, 233, 76, 50); ellipse(sinPointBottomX, sinPointBottomY, 20, 20); 
}
