/* Pong game. */

color BrightCyan = color(47, 196, 211); // the game is monochrome

int FieldWidth = 720;
int FieldHeight = 460;
float FieldBorder = 20;
float WallThickness = 10;     

float HalfPaddleHeight = 30;
float HalfPaddleWidth = 5;
float PaddleSpeed = 5;

float LeftPaddleX = FieldBorder;
float LeftPaddleY = FieldHeight / 2;
float LeftPaddle_dY = 0;
int LeftPaddleState = 0;
int NO_MOVEMENT = 0;
int MOVING_UP_BY_KEY = -1;                    
int MOVING_DOWN_BY_KEY = 1;
int MOVING_UP_BY_INERTIA = -2;
int MOVING_DOWN_BY_INERTIA = 2;
float Inertia = 0.1;                 

float RightPaddleX = FieldWidth - FieldBorder;
float RightPaddleY = FieldHeight / 2;
float RightPaddle_dY = 0;

float HalfBallWidth = 5;
float BallCenterX = FieldWidth / 2;
float BallCenterY = FieldHeight / 2;
float BallLeftX;
float BallRightX;
float BallTopY;
float BallBottomY;
float Ball_dX = 5;
float Ball_dY = 0;

int RESTING = 0;
int MOVING = 1;
int BallState = RESTING;
int NumRestFrames = 60; // the ball will rest for this long at the beginning of each round                
int CurrentRestFrame = 0;     
        
float SpeedCounter = -PaddleSpeed;

int LeftScore = 0;
int RightScore = 0;

int LEFT = 0;
int TOP = 1;
int RIGHT = 2;
int BOTTOM = 3;

void setup() {
  size(FieldWidth, FieldHeight);
  background(0); // black  
}

void keyTyped() {
  if (key == 'w') { // go up
    LeftPaddle_dY = -PaddleSpeed;
    LeftPaddleState = MOVING_UP_BY_KEY;
  } else if (key == 's') { // go down
    LeftPaddle_dY = PaddleSpeed;
    LeftPaddleState = MOVING_DOWN_BY_KEY;
  }
}

void keyReleased() {
  if (key == 'w') { // up
    LeftPaddleState = MOVING_UP_BY_INERTIA;
  } else if (key == 's') {    
    LeftPaddleState = MOVING_DOWN_BY_INERTIA;
  }
}

void draw() {  
  background(0);
  drawWalls();
  drawMiddleLine();
  drawPaddles();
  drawBall();
  logic();
}

void logic() {
  handleSpeedCounter();
  calculateBallSides();
  handleCollisions();
  handleBallState();
  moveBall();  
  moveLeftPaddle();
  moveRightPaddle();
  checkEndRound();
}

void handleSpeedCounter() {
  SpeedCounter += 0.5;
  if (SpeedCounter > PaddleSpeed) {
    SpeedCounter = -PaddleSpeed;
  }
}

void calculateBallSides() {
  BallLeftX = BallCenterX - HalfBallWidth;
  BallRightX = BallCenterX + HalfBallWidth;
  BallTopY = BallCenterY - HalfBallWidth;
  BallBottomY = BallCenterY + HalfBallWidth;
}

void handleBallState() {
  if (BallState == RESTING) {
    ++CurrentRestFrame;
    if (CurrentRestFrame >= NumRestFrames) {
      BallState = MOVING;
      CurrentRestFrame = 0;
    }
  }
}

void moveBall() {  
  if (BallState == MOVING) {
    BallCenterX += Ball_dX;
    BallCenterY += Ball_dY;  
  }
}

void moveLeftPaddle() {
  slowDownLeftPaddleIfKeyReleased();
  LeftPaddleY += LeftPaddle_dY;    
}

void slowDownLeftPaddleIfKeyReleased() {
  if ((LeftPaddleState == MOVING_UP_BY_INERTIA) && (LeftPaddle_dY < 0)) {
      LeftPaddle_dY += Inertia;
      if (LeftPaddle_dY > 0) {
        LeftPaddle_dY = 0;
      }
  } else if ((LeftPaddleState == MOVING_DOWN_BY_INERTIA) && (LeftPaddle_dY > 0)) {
      LeftPaddle_dY -= Inertia;
      if (LeftPaddle_dY < 0) {
        LeftPaddle_dY = 0;
      }
  }
}

void moveRightPaddle() {
  if (ballIsMovingLeft() && (BallCenterX < width / 2)) {
    if (rightPaddleIsMovingUp()) {
      RightPaddle_dY += Inertia;
    } else if (rightPaddleIsMovingDown()) {
      RightPaddle_dY -= Inertia;
    }
    return;
  } 
  
  rightPaddleFollowBall();
  calculateBallSides();
  if (ballCollidesRightPaddleNextFrame()) {
    RightPaddle_dY = SpeedCounter;    
    moveRightPaddleAwayFromWalls();
  }
  
  clampRightPaddleSpeed();
  RightPaddleY += RightPaddle_dY;
}

void rightPaddleFollowBall() {
  if (BallCenterY < RightPaddleY) {
    RightPaddle_dY = BallCenterY - RightPaddleY + 0.1;     
  } else if (BallCenterY > RightPaddleY) {
    RightPaddle_dY = BallCenterY - RightPaddleY - 0.1;    
  }
}

boolean ballCollidesRightPaddleNextFrame() {
  return ballCollidesRect(RightPaddleX, RightPaddleY, HalfPaddleWidth, HalfPaddleHeight);
}

void moveRightPaddleAwayFromWalls() {
  if (rightPaddleCollidesTopWall() && rightPaddleIsMovingUp()) {      
      RightPaddle_dY = -RightPaddle_dY;      
  } else if (rightPaddleCollidesBottomWall() && rightPaddleIsMovingDown()) {      
      RightPaddle_dY = -RightPaddle_dY;      
  }
}

boolean rightPaddleIsMovingUp() {
  return RightPaddle_dY < 0;
}

boolean rightPaddleIsMovingDown() {
  return RightPaddle_dY > 0;
}

void clampRightPaddleSpeed() {
  if (RightPaddle_dY < -PaddleSpeed) {
    RightPaddle_dY = -PaddleSpeed;
  } else if (RightPaddle_dY > PaddleSpeed) {
    RightPaddle_dY = PaddleSpeed;
  }
}

void handleCollisions() {
  handleLeftPaddleWallCollision();
  handleRightPaddleWallCollision();
  handleBallCollisions();
}

void handleLeftPaddleWallCollision() {
  if (leftPaddleCollidesTopWall() || leftPaddleCollidesBottomWall()) {
    LeftPaddleY -= LeftPaddle_dY;
    LeftPaddle_dY = 0;    
  }
}

void handleRightPaddleWallCollision() {
  if (rightPaddleCollidesTopWall() || rightPaddleCollidesBottomWall()) {
    RightPaddleY -= RightPaddle_dY;
    RightPaddle_dY = 0;
  }
}

boolean leftPaddleCollidesTopWall() {
  return LeftPaddleY - HalfPaddleHeight - WallThickness <= 0;
}

boolean rightPaddleCollidesTopWall() {
  return RightPaddleY - HalfPaddleHeight - WallThickness <= 0;
  
}

boolean leftPaddleCollidesBottomWall() {
  return LeftPaddleY + HalfPaddleHeight + WallThickness >= height;
}

boolean rightPaddleCollidesBottomWall() {
  return RightPaddleY + HalfPaddleHeight + WallThickness >= height;
}

void handleBallCollisions() {
  handleBallLeftPaddleCollision();
  handleBalRightPaddleCollision();
  handleBallWallsCollisions();
}

void handleBallLeftPaddleCollision() {  
  if (ballCollidesRect(LeftPaddleX, LeftPaddleY, HalfPaddleWidth, HalfPaddleHeight)) {
    handleBallRectCollision(LeftPaddleX, LeftPaddleY, HalfPaddleWidth, HalfPaddleHeight);
    Ball_dY += LeftPaddle_dY;
  }    
}

void handleBalRightPaddleCollision() {  
  if (ballCollidesRect(RightPaddleX, RightPaddleY, HalfPaddleWidth, HalfPaddleHeight)) {
    handleBallRectCollision(RightPaddleX, RightPaddleY, HalfPaddleWidth, HalfPaddleHeight);
    Ball_dY += RightPaddle_dY;    
  }    
}

boolean ballCollidesRect(float centerX, float centerY,
                         float halfLengthX, float halfLengthY) {
  float rectTopY = centerY - halfLengthY;
  float rectBottomY = centerY + halfLengthY;
  float rectLeftX = centerX - halfLengthX;
  float rectRightX = centerX + halfLengthX;    
  
  return !((BallLeftX > rectRightX) || (BallRightX < rectLeftX) ||
           (BallTopY > rectBottomY) || (BallBottomY < rectTopY));
}

void handleBallRectCollision(float centerX, float centerY,
                             float halfLengthX, float halfLengthY) {  
  moveBallBackOneFrame();  
  
  int side = ballCollideWhichRectSide(centerX, centerY, halfLengthX, halfLengthY);
  if ((side == LEFT) && ballIsMovingRight()) {
    Ball_dX *= -1;
  } else if ((side == RIGHT) && ballIsMovingLeft()) {
    Ball_dX *= -1;
  } else if ((side == TOP) && ballIsMovingDown()) {
    Ball_dY *= -1;
  } else if ((side == BOTTOM) && ballIsMovingUp()) {        
    Ball_dY *= -1;
  }  
}

int ballCollideWhichRectSide(float centerX, float centerY,
                             float halfLengthX, float halfLengthY) {
  float d = halfLengthY - halfLengthX;
  float x = BallCenterX - Ball_dX - centerX;
  float y = BallCenterY - Ball_dY - centerY;
  
  if ((x < 0) && (x - d <= y) && (y <= -x + d)) {
    return LEFT;
  } else if ((y < 0) && (y + d <= x) && (x <= -d - y)) {
    return TOP;
  } else if ((x > 0) && (-x - d <= y) && (y <= x + d)) {
    return RIGHT;
  } else if ((y > 0) && (d - y <= x) && (x <= y - d)) {
    return BOTTOM;
  }
  
  return -1;
}

boolean ballIsMovingLeft() {
  return Ball_dX < 0;
}

boolean ballIsMovingRight() {
  return Ball_dX > 0;
}

boolean ballIsMovingUp() {
  return Ball_dY < 0;
}

boolean ballIsMovingDown() {
  return Ball_dY > 0;
}

void handleBallWallsCollisions() {  
  if (ballHitsBottomWall() || ballHitsTopWall()) {    
    moveBallBackOneFrame();
    Ball_dY *= -1;   
  }  
}

boolean ballHitsTopWall() {
  return ballIsInField() && (BallTopY <= WallThickness);
}

boolean ballHitsBottomWall() {
  return ballIsInField() && (BallBottomY >= height - WallThickness);
}

boolean ballIsInField() {
  return ballIsOutOfField() == -1;
}

int ballIsOutOfField() {
  if (BallRightX < 0) {
    return LEFT;
  } else if (BallLeftX > width) {
    return RIGHT;
  } else {
    return -1;
  }
}

void moveBallBackOneFrame() {
  BallCenterX -= Ball_dX;
  BallCenterY -= Ball_dY;
}


void checkEndRound() {
  int sideOut = ballIsOutOfField();
  if (sideOut != -1) {        
    if (sideOut == LEFT) {
      ++RightScore;
    } else if (sideOut == RIGHT) {
      ++LeftScore;
    }
    printScore();
    resetGame();
  }
}

void printScore() {
  println("Score: " + LeftScore + " - " + RightScore);
}

void resetGame() {
  resetBall();
  resetPaddles();
}

void resetBall() {
  BallCenterX = width / 2;
  BallCenterY = height / 2;
  Ball_dX = 5;   
  Ball_dY = 0;
  BallState = RESTING;
}

void resetPaddles() {
  RightPaddleY = LeftPaddleY = height / 2;
  LeftPaddleState = NO_MOVEMENT;
  LeftPaddle_dY = 0;
  RightPaddle_dY = 0;
}

void drawPaddles() {
  drawPaddle(LeftPaddleX, LeftPaddleY);
  drawPaddle(RightPaddleX, RightPaddleY);
}

void drawPaddle(float centerX, float centerY) {
  fill(BrightCyan);
  noStroke();
  rect(centerX - HalfPaddleWidth, centerY - HalfPaddleHeight,
       HalfPaddleWidth * 2, HalfPaddleHeight * 2);
}

void drawBall() {
  fill(BrightCyan);
  noStroke();
  rect(BallCenterX - HalfBallWidth, BallCenterY - HalfBallWidth,
       HalfBallWidth * 2, HalfBallWidth * 2);  
}

void drawWalls() {
  fill(BrightCyan);
  noStroke();
  rect(0, 0, width, WallThickness);
  rect(0, height, width, -WallThickness);
}

void drawMiddleLine() {
  stroke(BrightCyan);
  strokeWeight(3);
  line(width / 2,   0, width / 2,  20);
  line(width / 2,  40, width / 2,  60);
  line(width / 2,  80, width / 2, 100);
  line(width / 2, 120, width / 2, 140);
  line(width / 2, 160, width / 2, 180);
  line(width / 2, 200, width / 2, 220);
  line(width / 2, 240, width / 2, 260);
  line(width / 2, 280, width / 2, 300);
  line(width / 2, 320, width / 2, 340);
  line(width / 2, 360, width / 2, 380);
  line(width / 2, 400, width / 2, 420);
  line(width / 2, 440, width / 2, 460);
}
