// HD 720p resolution :)
int WindowWidth = 1080;
int WindowHeight = 720;

// All base colors
color Background = color(50, 50, 50, 255);
color AlmostWhite = color(250);

// Control panel
float CpWidth = 250;
float CpInnerBorder = 10;
float CpInnerWidth = CpWidth - 2 * CpInnerBorder;
float CpInnerRightEdge = CpWidth - CpInnerBorder;
float CpBarHeight = 20;
float CpBarSpacing = 5;

// Stroke states
// Each stroke draws a number of "sub-shapes" of the same type,
// with decreasing size and varying colors.
int NumSubShapes = 20; 
int NumSubShapesMin = 1;
int NumSubShapesMax = 40;
float ShapeSize = 50;
float ShapeSizeMin = 1;
float ShapeSizeMax = CpInnerWidth;
float InnerShapeSize = 30;

// Color pickers
int RedVal = 50; // 0 - 255
int GreenVal = 129; // 0 - 255
int BlueVal = 166; // 0 - 255
int AlphaVal = 255; // 0 - 255
float RedBarX = CpInnerBorder;
float RedBarY = CpInnerBorder;
float GreenBarX = CpInnerBorder;
float GreenBarY = RedBarY + CpBarHeight + CpBarSpacing;
float BlueBarX = CpInnerBorder;
float BlueBarY = GreenBarY + CpBarHeight + CpBarSpacing;
float AlphaBarX = CpInnerBorder;
float AlphaBarY = BlueBarY + CpBarHeight + CpBarSpacing;

// Preview
float PreviewX = CpWidth / 2.0;
float PreviewY = AlphaBarY + CpBarHeight + CpBarSpacing + CpInnerBorder + ShapeSizeMax / 2;

// Inner Shape bar
float InnerShapeBarX = CpInnerBorder;
float InnerShapeBarY = PreviewY + CpBarSpacing + CpInnerBorder + ShapeSizeMax / 2;

// Number of sub-shapes bar
float NumSubShapesBarX = CpInnerBorder;
float NumSubShapesBarY = InnerShapeBarY + CpBarHeight + CpBarSpacing;

int DragCounterMin = 1;
int DragCounterMax = 5;
int DragCounterLimit = 60;
int DragCounter = 1;

// Controls the DragCounterMax
float DragCounterBarX = CpInnerBorder;
float DragCounterBarY = NumSubShapesBarY + CpBarHeight + CpBarSpacing;

// Auto changing Shape's size
boolean AutoShapeSize = false;
float SizeMultiplierMin = 1.0;
float SizeMultiplierMax = 2.0;
float SizeMultiplier = SizeMultiplierMin;
float SizeModifier = 1.0;

float SizeMultiplierBarX = CpInnerBorder;
float SizeMultiplierBarY = DragCounterBarY + CpBarHeight + CpBarSpacing;

int SHAPE_MIN = 0;
int SHAPE_CIRCLE = 0;
int SHAPE_SQUARE = 1;
int SHAPE_TRIANGLE = 2;
int SHAPE_HEART = 3;
int SHAPE_MAX = 3;
int Shape = SHAPE_CIRCLE;

color ShapeColor = color(RedVal, GreenVal, BlueVal, AlphaVal);

void setup() {
  size(WindowWidth, WindowHeight);
  smooth();
  background(Background);
}

void draw() {
  updateColor();
  drawControlPanel();  
}

void mouseClicked() {  
  if (mouseIsInControlPanel()) {
    control();
  } else {
    drawShape(mouseX, mouseY, false);
  }
}

void mouseDragged() {
  if (mouseIsInControlPanel()) {
    control();
  } else {
    if (DragCounter == DragCounterMin) {
      updateShapeSizeModifier();
      drawShape(mouseX, mouseY, false);
    }
    DragCounter = DragCounter >= DragCounterMax ? DragCounterMin : ++DragCounter;
  }
}

void mouseReleased() {
  DragCounter = DragCounterMin;
  SizeModifier = 1.0;  
}

void keyPressed() {
  if (key == 's') {
    ++Shape;
    if (Shape > SHAPE_MAX) {
      Shape = SHAPE_MIN;
    }
  }
}

void updateColor() {
  ShapeColor = color(RedVal, GreenVal, BlueVal, AlphaVal);
}

boolean mouseIsInControlPanel() {
  return mouseX >= 0 && mouseX <= CpWidth;
}

void control() {
  if (mouseIsInRedBar()) {    
    handleMouseInRedBar();
  } else if (mouseIsInGreenBar()) {
    handleMouseInGreenBar();
  } else if (mouseIsInBlueBar()) {
    handleMouseInBlueBar();
  } else if (mouseIsInAlphaBar()) {
    handleMouseInAlphaBar();
  } else if (mouseIsInPreview()) {
    handleMouseInPreview();
  } else if (mouseIsInInnerShape()) {
    handleMouseInInnerShape();
  } else if (mouseIsInNumSubShapesBar()) {
    handleMouseInNumSubShapesBar();
  } else if (mouseIsInDragCounterBar()) {
    handleMouseInDragCounterBar();
  } else if (mouseIsInSizeMultiplierBar()) {
    handleMouseInSizeMultiplierBar();
  }
}

boolean mouseIsInRedBar() {
  return mouseIsInRect(RedBarX, RedBarY, CpInnerWidth, CpBarHeight);
}

void handleMouseInRedBar() {
  RedVal = int(map(mouseX, RedBarX, CpInnerRightEdge, 0, 255));            
}

boolean mouseIsInGreenBar() {
  return mouseIsInRect(GreenBarX, GreenBarY, CpInnerWidth, CpBarHeight);
}

void handleMouseInGreenBar() {
  GreenVal = int(map(mouseX, GreenBarX, CpInnerRightEdge, 0, 255));        
}

boolean mouseIsInBlueBar() {
  return mouseIsInRect(BlueBarX, BlueBarY, CpInnerWidth, CpBarHeight);
}

void handleMouseInBlueBar() {
  BlueVal = int(map(mouseX, BlueBarX, CpInnerRightEdge, 0, 255));
}

boolean mouseIsInAlphaBar() {
  return mouseIsInRect(AlphaBarX, AlphaBarY, CpInnerWidth, CpBarHeight);
}

void handleMouseInAlphaBar() {
  AlphaVal = int(map(mouseX, AlphaBarX, CpInnerRightEdge, 0, 255));
}

boolean mouseIsInPreview() {
  return mouseIsInRect(PreviewX - ShapeSizeMax / 2, PreviewY - ShapeSizeMax / 2,
                       ShapeSizeMax, ShapeSizeMax);
}

// Resizes the Shape
void handleMouseInPreview() {
  ShapeSize = min(2 * dist(mouseX, mouseY, PreviewX, PreviewY), ShapeSizeMax);  
}

boolean mouseIsInInnerShape() {  
  return mouseIsInRect(InnerShapeBarX, InnerShapeBarY, CpInnerWidth, CpBarHeight);
}

void handleMouseInInnerShape() {
  InnerShapeSize = map(mouseX, InnerShapeBarX, CpInnerRightEdge, ShapeSizeMin, ShapeSizeMax);         
}

boolean mouseIsInNumSubShapesBar() {  
  return mouseIsInRect(NumSubShapesBarX, NumSubShapesBarY, CpInnerWidth, CpBarHeight);
}

void handleMouseInNumSubShapesBar() {
  NumSubShapes = int(map(mouseX, NumSubShapesBarX, CpInnerRightEdge, NumSubShapesMin, NumSubShapesMax));         
}

boolean mouseIsInDragCounterBar() {
  return mouseIsInRect(DragCounterBarX, DragCounterBarY, CpInnerWidth, CpBarHeight);
}

void handleMouseInDragCounterBar() {
  DragCounterMax = int(map(mouseX, DragCounterBarX, CpInnerRightEdge, DragCounterMin, DragCounterLimit));
}

boolean mouseIsInSizeMultiplierBar() {
  return mouseIsInRect(SizeMultiplierBarX, SizeMultiplierBarY, CpInnerWidth, CpBarHeight);
}

void handleMouseInSizeMultiplierBar() {
  SizeMultiplier = map(mouseX, SizeMultiplierBarX, CpInnerRightEdge, SizeMultiplierMin, SizeMultiplierMax);
}

// Returns whether the mouse is inside a rectangle specified by (x1, y1, x2, y2).
// The meaning of the arguments is the same as those of the default rect() mode.
boolean mouseIsInRect(float x1, float y1, float x2, float y2) {
  return mouseX >= x1 && mouseX <= x1 + x2 &&
         mouseY >= y1 && mouseY <= y1 + y2;
}

void updateShapeSizeModifier() {  
  SizeModifier *= SizeMultiplier;  
}

void drawControlPanel() {  
  drawPanel();
  drawColorBars();
  drawPreview();
  drawInnerShapeBar();
  drawNumSubShapesBar();
  drawDragCounterBar();
  drawSizeMultiplierBar();
}

void drawPanel() {
  strokeWeight(1);
  stroke(250);
  fill(Background);
  rect(0, 0, CpWidth, height);  
}

void drawColorBars() {  
  strokeWeight(1);
  stroke(250);
  // red bar
  fill(RedVal, 0, 0);
  rect(RedBarX, RedBarY,
       map(RedVal, 0, 255, 0, CpInnerWidth), CpBarHeight);
  // green bar
  fill(0, GreenVal, 0);
  rect(GreenBarX, GreenBarY,       
       map(GreenVal, 0, 255, 0, CpInnerWidth), CpBarHeight);
  // blue bar
  fill(0, 0, BlueVal);
  rect(BlueBarX, BlueBarY,
       map(BlueVal, 0, 255, 0, CpInnerWidth), CpBarHeight);
  // alpha bar
  fill(AlphaVal, AlphaVal, AlphaVal, AlphaVal);
  rect(AlphaBarX, AlphaBarY,
       map(AlphaVal, 0, 255, 0, CpInnerWidth), CpBarHeight);
}

void drawPreview() {
  drawShape(PreviewX, PreviewY, true);
}

void drawInnerShapeBar() {
  fill(ShapeColor);
  rect(InnerShapeBarX, InnerShapeBarY,
       map(InnerShapeSize, ShapeSizeMin, ShapeSizeMax, 0, CpInnerWidth), CpBarHeight);
}

void drawNumSubShapesBar() {
  fill(ShapeColor);
  rect(NumSubShapesBarX, NumSubShapesBarY,
       map(NumSubShapes, NumSubShapesMin, NumSubShapesMax, 0, CpInnerWidth), CpBarHeight);
}

void drawDragCounterBar() {
  fill(AlmostWhite);
  rect(DragCounterBarX, DragCounterBarY,
       map(DragCounterMax, DragCounterMin, DragCounterLimit, 0, CpInnerWidth), CpBarHeight);
}

void drawSizeMultiplierBar() {
  fill(AlmostWhite);
  rect(SizeMultiplierBarX, SizeMultiplierBarY,
       map(SizeMultiplier, SizeMultiplierMin, SizeMultiplierMax, 0, CpInnerWidth), CpBarHeight);
}

void drawShape(float x, float y, boolean preview) {
  float sizeStep = (ShapeSize - InnerShapeSize) / (NumSubShapes - 1);
  noStroke();
  for (int i = 0; i < NumSubShapes; ++i) {    
    fill(lerpColor(Background, ShapeColor, float(i) / NumSubShapes), alpha(ShapeColor));    
    float cSize = ShapeSize - sizeStep * i;
    if (preview) {
      switch (Shape) {
        case 0:
          drawCircle(x, y, constrain(cSize, ShapeSizeMin, ShapeSizeMax));
          break;
        case 1:
          drawSquare(x, y, constrain(cSize, ShapeSizeMin, ShapeSizeMax));
          break;        
        case 2:
          drawTriangle(x, y, constrain(cSize, ShapeSizeMin, ShapeSizeMax));
          break;
        case 3:
          drawHeart(x, y, constrain(cSize, ShapeSizeMin, ShapeSizeMax));
          break;
      }
    } else {
      switch (Shape) {
        case 0:
          drawCircle(x, y, SizeModifier * constrain(cSize, ShapeSizeMin, ShapeSizeMax));
          break;
        case 1:
          drawSquare(x, y, SizeModifier * constrain(cSize, ShapeSizeMin, ShapeSizeMax));
          break;
        case 2:
          drawTriangle(x, y, SizeModifier * constrain(cSize, ShapeSizeMin, ShapeSizeMax));
          break;
        case 3:
          drawHeart(x, y, SizeModifier * constrain(cSize, ShapeSizeMin, ShapeSizeMax));
          break;        
      }      
    }
  }  
}

void drawCircle(float x, float y, float cSize) {
  ellipse(x, y, cSize, cSize);
}

void drawSquare(float x, float y, float cSize) {
  float R = cSize / 2;
  float r = 10;
  float dBig = sqrt(sq(R) / 2);
  float dSmall = dBig - r;  
  // we could draw fewer shapes, but we want to achieve zero overlap here.
  rect(x - dSmall, y - dBig, dSmall * 2, r); // top one
  rect(x - dSmall, y + dSmall, dSmall * 2, r); // bottom one
  rect(x - dBig, y - dSmall, r, dSmall * 2); // left one
  rect(x + dSmall, y - dSmall, r, dSmall * 2); // right one
  rect(x - dSmall, y - dSmall, dSmall * 2, dSmall * 2); // center one  
  arc(x - dSmall, y - dSmall, r * 2, r * 2, radians(180), radians(270)); // top-left
  arc(x + dSmall, y - dSmall, r * 2, r * 2, radians(270), radians(360)); // top-right
  arc(x - dSmall, y + dSmall, r * 2, r * 2, radians(90), radians(180)); // bottom-left
  arc(x + dSmall, y + dSmall, r * 2, r * 2, radians(0), radians(90)); // bottom-right
}

void drawTriangle(float x, float y, float cSize) {
  float R = cSize / 2;
  float r = 5;
  float D = R - (2*r);
  float DSmall = D / 2;
  float DBig = sqrt(sq(D) - sq(DSmall));  
  float dSmall = r / 2;
  float dBig = sqrt(sq(r) - sq(dSmall));  
  float xx1 = x;
  float yy1 = y  - D;
  float xx2 = x + DBig;
  float yy2 = y + DSmall;
  float xx3 = x - DBig;
  float yy3 = y + DSmall;
  triangle(xx1, yy1, xx2, yy2, xx3, yy3);
  quad(xx1, yy1, xx1-dBig, yy1-dSmall, xx3-dBig, yy3-dSmall, xx3, yy3); // left
  quad(xx1, yy1, xx1+dBig, yy1-dSmall, xx2+dBig, yy2-dSmall, xx2, yy2); // right
  quad(xx3, yy3, xx3, yy3 + r, xx2, yy2 + r, xx2, yy2); // bottom
  arc(xx1, yy1, r * 2, r * 2, radians(210), radians(330)); // top
  arc(xx3, yy3, r * 2, r * 2, radians(90), radians(210)); // left
  arc(xx2, yy2, r * 2, r * 2, radians(330), radians(450)); // right
}

// the parametric equation is taken from http://mathworld.wolfram.com/HeartCurve.html
void drawHeart(float x, float y, float cSize) {
  noStroke();
  beginShape();
    for (float t = 0; t <= TWO_PI; t += 0.05) {
      float sint = sin(t);      
      float xx = x + (16 * sint * sint * sint) / 30 * cSize;
      float yy = y - (13 * cos(t) - 5*cos(2*t) - 2*cos(3*t) - cos(4*t)) / 30 * cSize;
      vertex(xx, yy);    
    }
  endShape(CLOSE);
}