int WindowWidth = 1080;
int WindowHeight = 720;

float[][] points1 = createAString(50, 10, 30);
float[] curveX1 = generateBezierCurve(points1[0]);
float[] curveY1 = generateBezierCurve(points1[1]);

float[][] points2 = createAString(50, 10, 30);
float[] curveX2 = generateBezierCurve(points2[0]);
float[] curveY2 = generateBezierCurve(points2[1]);

float[] curveX = new float[curveX1.length];
float[] curveY = new float[curveX1.length];

float t = 0;
float rotation = 0;

void setup() {
  smooth();
  size(WindowWidth, WindowHeight);    
  arrayCopy(curveX1, curveX);
  arrayCopy(curveY1, curveY);
}

void draw() {
  moveStuffs();
  drawStuffs();  
}

void moveStuffs() {
/*
  float[] copyX = points[0];
  float[] copyY = points[1];
  for (int i = 0; i < points[0].length; ++i) {
    //int j = (i + int(random(0, points[0].length))) % points[0].length;
    //points[0][i] += random(-1, 1);
    //points[1][i] += random(-1, 1);
    float theta = atan2(points[1][i], points[0][i]);
    float r = mag(points[1][i], points[0][i]);
    r += random(-5, 5);
    //theta += radians(random(-1, 1));
    points[0][i] = r * cos(theta);
    points[1][i] = r * sin(theta);
  }
  curveX = generateBezierCurve(points[0]);
  curveY = generateBezierCurve(points[1]);
  */
  if (t <= 1) {
    for (int i = 0; i < curveX.length; ++i) {
      if (i % 3 == 0) {
        curveX[i] = lerp(curveX1[i], curveX2[i], t);
        curveY[i] = lerp(curveY1[i], curveY2[i], t);    
        int prev = i > 0 ? i - 1 : curveX.length - 1;
        int next = i + 1;
        curveX[prev] = lerp(curveX1[prev], curveX2[prev], t);
        curveY[prev] = lerp(curveY1[prev], curveY2[prev], t);   
        curveX[next] = curveX[i] * 2 - curveX[prev];
        curveY[next] = curveY[i] * 2 - curveY[prev];
      }
    }
    t += 0.1;
  } else {    
    points1 = points2;
    curveX1 = curveX2;
    curveY1 = curveY2;
    points2 = createAString(50, 10, 30);
    curveX2 = generateBezierCurve(points2[0]);
    curveY2 = generateBezierCurve(points2[1]);    
    t = 0;
    //noLoop();
  }
}

float[][] createAString(float radius, float deviation, int numPoints) {
  float eps = TWO_PI / numPoints / 4;  
  float[] r = new float[numPoints];
  float[] theta = new float[numPoints];
  for (int i = 0; i < numPoints; ++i) {
    r[i] = random(radius - deviation, radius + deviation);
    theta[i] = random(i * TWO_PI / numPoints + eps, (i + 1) * TWO_PI / numPoints - eps);
  }
  float[][] points = new float[2][];
  points[0] = new float[numPoints];
  points[1] = new float[numPoints];
  for (int i = 0; i < numPoints; ++i) {
    points[0][i] = r[i] * cos(theta[i]);
    points[1][i] = r[i] * sin(theta[i]);
  }
  return points;
}

/* Given the knots, generate the handles and put all of them together in the returned array.
The result can then be drawn as one closed, smooth Bezier curve going through all the knots. */
float[] generateBezierCurve(float[] knots) {
  float[] myCurve = new float[knots.length * 3];
  for (int i = 0; i < knots.length; ++i) {
    int j = i * 3;
    myCurve[j] = knots[i];
    int next = i < knots.length - 1 ? i + 1 : 0;
    int prev = i > 0 ? i - 1 : knots.length - 1;
    float d = knots[next] - knots[prev];
    int nextJ = j + 1;
    int prevJ = j > 0 ? j - 1 : knots.length * 3 - 1;
    myCurve[nextJ] = myCurve[j] + d / 4;
    myCurve[prevJ] = myCurve[j] - d / 4;
  }
  return myCurve;
}

void drawStuffs() {
  background(50);
  noFill();
  
  pushMatrix();
    translate(WindowWidth / 2, WindowHeight / 2);
    rotate(rotation);
    rotation += radians(random(0, 0.5));
    for (int i = 20; i >= 10; --i) { 
      strokeWeight(i);
      stroke(lerpColor(color(50, 50, 50, 255), color(255, 0, 0, 255), (20.0 - i) / 10));
      drawBezierCurve(curveX, curveY);    
    }
    strokeWeight(2);
    stroke(255);
    drawBezierCurve(curveX, curveY);    
    
    
  popMatrix();
}

/* Draws a closed Bezier curve with the data from curveX and curveY. */
void drawBezierCurve(float[] curveX, float[] curveY) {
  
  beginShape();
    vertex(curveX[0], curveY[0]);
    for (int i = 1; i < curveX.length; i += 3) {
      int j0 = i % curveX.length;
      int j1 = (i + 1) % curveX.length;
      int j2 = (i + 2) % curveX.length;
      bezierVertex(curveX[j0], curveY[j0], curveX[j1], curveY[j1], curveX[j2], curveY[j2]);
    }
  endShape(CLOSE);
  for (int i = 0; i < curveX.length; i += 3) {
    //ellipse(curveX[i], curveY[i], 10, 10);
  }
}

/* Draws a closed Catmull-Rom curve with the data from curveX and curveY. */
void drawCRCurve(float[] curveX, float[] curveY) {
  noFill();
  strokeWeight(2);
  stroke(0, 0, 255);
  beginShape();
    for (int i = 0; i < curveX.length + 3; ++i) {
      int j = i % curveX.length;
      curveVertex(curveX[j], curveY[j]);
    }
  endShape(CLOSE);
}

boolean noLoop = true;
void keyTyped() {
  // if (key == ' ') {
    // noLoop = !noLoop;
  // }
  // if (noLoop) {
    // noLoop();
  // } else {
    // loop();
  // }
  loop();
}