/*
In this scene, a blurred image of some flowers and leaves are shown.
The image then becomes clearer and clearer.
Then some text appears (in a smooth left-to-right way).
The text seems to carry some deep meaning. In fact it's just cryptic and meaningless.
When the text is all written down, the whole scene turns into white.
*/

PImage[] FlowerImgs = new PImage[8];
PFont FlowerFont;

String[] FlowerText = new String[] {
  "There is only one path to Heaven",
  "On Earth, we call it love",
  "'Cause love itself is just as innocent",
  "        as roses in May",
  "It is pure white",
  "        just like sin                                "
};

boolean FlowerSceneInit = false;

float FlowerImgIdx = FlowerImgs.length - 0.000001;

int FlowerTextIdx = 0;

int FlowerTextAscent;
int FlowerTextDescent;
int FlowerTextWidth;
int FlowerTextHeight;
color[] FlowerTextMask;
float MaskT = 0;

color[] ScreenLastFrame;

void drawFlowerScene() {  
  if (!FlowerSceneInit) {
    initFlowerScene();
    FlowerSceneInit = true;
  }
  
  if (FlowerImgIdx >= 1) { 
    /* lerp between the flower images */
    loadPixels();
    int img1 = int(FlowerImgIdx);
    int img2 = img1 - 1;        
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < width; ++j) {
        int pix = i * width + j;
        pixels[pix] = lerpColor(FlowerImgs[img1].pixels[pix], FlowerImgs[img2].pixels[pix], 1 - (FlowerImgIdx - img1));
      }
    }
    FlowerImgIdx -= 0.01;
    updatePixels();
  } else if (FlowerTextIdx < FlowerText.length) {
    /* display the text using a masking technique */
    /* save last frame */
    loadPixels();
    arrayCopy(pixels, ScreenLastFrame);    
    /* draw the mask using a moving rectangle */  
    background(0);
    fill(230, 193, 176);
    int x = 120;
    int y = 400 + FlowerTextHeight * FlowerTextIdx;
    text(FlowerText[FlowerTextIdx], x, y);
    fill(0, 0, 0 ,255);  
    rect(map(MaskT, 0, 1, x, x + FlowerTextWidth + 10), y - FlowerTextAscent, FlowerTextWidth, FlowerTextHeight);
    MaskT += 0.005;
    if (MaskT > 1) {
      MaskT = 0;
      ++FlowerTextIdx;
      if (FlowerTextIdx < FlowerText.length) {
        FlowerTextWidth = int(textWidth(FlowerText[FlowerTextIdx]) + 0.5);
        FlowerTextMask = new color[FlowerTextWidth * (FlowerTextAscent + FlowerTextDescent)]; 
      }
    }    
    /* copy the mask from screen */
    loadPixels();
    for (int i = 0; i < FlowerTextHeight; ++i) {
      for (int j = 0; j < FlowerTextWidth; ++j) {
        FlowerTextMask[i * FlowerTextWidth + j] = pixels[(i + y - FlowerTextAscent) * width + (j + x)];
      }
    }    
    /* draw the last frame */
    arrayCopy(ScreenLastFrame, pixels);     
    /* draw the text */    
    for (int i = 0; i < FlowerTextHeight; ++i) {
      for (int j = 0; j < FlowerTextWidth; ++j) {
        int pix = i * FlowerTextWidth + j;
        if (red(FlowerTextMask[pix]) > 0 && green(FlowerTextMask[pix]) > 0) {
          pixels[(i + y) * width + (j + x)] = FlowerTextMask[i * FlowerTextWidth + j];
        }
      }
    }      
    updatePixels();
  } else {
    /* turn random pixels to white and end the scene */
    loadPixels();
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < width; ++j) {
        if (random(0, 1) < 0.5) {
          continue;
        }
        int ii = i + int(random(-2, 2));
        int jj = j + int(random(-2, 2));
        int idx = i * width + j;
        if (red(pixels[idx]) == 255 && green(pixels[idx]) == 255 && blue(pixels[idx]) == 255) {
          continue;
        }        
        if (ii > 0 && jj > 0 && ii < height && jj < width) {        
          pixels[i * width + j] = pixels[ii * width + jj];
        } else {
          pixels[i * width + j] = color(255);
        }
      }
    }
    updatePixels();
  }
}

void initFlowerScene() {
  textFont(FlowerFont);
  FlowerTextAscent = int(textAscent() + 0.5);
  FlowerTextDescent = int(textDescent() + 0.5);  
  FlowerTextHeight = FlowerTextAscent + FlowerTextDescent;
  FlowerTextWidth = int(textWidth(FlowerText[FlowerTextIdx]) + 0.5);
  FlowerTextMask = new color[FlowerTextWidth * (FlowerTextAscent + FlowerTextDescent)];
  ScreenLastFrame = new color[width * height];  
}