/*
In this scene, an image of a train is displayed.
Then some text (a quote from the Inception movie) will be displayed.
Then some text will come out of the train's chimney, they will go to the left and up.
Then the camera will zoom-in, and the screen fades to the second scene.
*/

PFont TrainTextFont;
PFont ChimneyTextFont;
PImage TrainImg;
PImage CloudMixImg;

String[] TrainText = new String[] {
  "You're waiting for a train.",
  "A train that will take you far away.",
  "You know where you hope the train will take you,",
  "but you can't be sure.",
  "Yet it doesn't matter."
};

String ChimneyText = "Heaven7";
float[] ChimneyTextDx = new float[ChimneyText.length()];
float[] ChimneyTextDy = new float[ChimneyText.length()];
float[] ChimneyTextS = new float[ChimneyText.length()]; // scale
float[] ChimneyTextX = new float[ChimneyText.length()];
float[] ChimneyTextY = new float[ChimneyText.length()];
float[] ChimneyTextSign = new float[ChimneyText.length()];

// Whether the scene's variables have been initialized
boolean TrainSceneInit = false;

// Which string is being processed.
int TextIndex = 0;
float TextOpacity = 0;

float WholeSceneScale = 1;

PVector[] TextBox;

void drawTrainScene() {
  if (!TrainSceneInit) {
    initTrainScene();
    TrainSceneInit = true;
  }
  
  if (TextIndex < TrainText.length) {
    /* show the quote */
    if (t < 0.5) {    
      showText();
      TextOpacity += 0.2;
      t += 0.003;
    } else if (t >= 0.5 && t < 1) {    
      discardText();
      t += 0.003;    
    } else if (t >= 1) {
      t = 0;
      ++TextIndex;
      TextOpacity = 0;
    }
    t += 0.001;
  } else if (t < 1.4) {
    /* show the chimney text */
    pushMatrix();
      if (t > 1) {
        scale(WholeSceneScale);
        WholeSceneScale += 0.005;
      }
      image(TrainImg, 0, 0);
      for (int i = 0; i < ChimneyText.length(); ++i) {
        animateChar(i);
      }
    popMatrix();
    t += 0.001;
  } else if (t >= 1.4 && t < 2.4) {
    /* zoom-in and fade to the second scene */
    pushMatrix();
      scale(WholeSceneScale);
      WholeSceneScale += 0.005;
      for (int i = 0; i < ChimneyText.length(); ++i) {
        animateChar(i);
      }
    popMatrix();
    loadPixels();
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < width; ++j) {
        int pix = i * width + j;
        pixels[pix] = lerpColor(pixels[pix], CloudMixImg.pixels[pix], t - 1.4);
      }
    }
    updatePixels();
    t += 0.001;
  } else {
    /* end of this scene */
    T = 1;
    t = 0;
  }
}

void initTrainScene() {  
  t = 0;
  image(TrainImg, 0, 0);  
}

/* The Inception quote will be faded-in sentence by sentence (specified by TextIndex). */
void showText() {
  if (TextIndex < TrainText.length) {
    fill(255, 255, 255, TextOpacity);
    textFont(TrainTextFont);
    textSize(60);
    text(TrainText[TextIndex], 100, 200);
    TextBox = getBoundingBoxOfString(TrainText[TextIndex], 100, 200);
  }
}

/* Discards text by blending each pixel with its left ones.
The text will appear to move to the right and then blends into the background. */
void discardText() {
  loadPixels();
  color[] clonedPix = new color[pixels.length];
  arrayCopy(pixels, clonedPix);
  float x = TextBox[0].x;
  float y = TextBox[0].y;
  float w = TextBox[1].x;
  float h = TextBox[1].y;
  for (int i = int(y); i < y + h; ++i) {
    for (int j = int(x); j < x + w + 50; ++j) {           
      int j1 = j - 1;
      int j2 = j - 2;
      int pix = i * width + j;
      int pix1 = i * width + j1;
      int pix2 = i * width + j2;
      float r = (6 * red(clonedPix[pix]) + 4 * red(clonedPix[pix1]) + red(clonedPix[pix2])) / 11.0;
      float g = (6 * green(clonedPix[pix]) + 4 * green(clonedPix[pix1]) + green(clonedPix[pix2])) / 11.0;
      float b = (6 * blue(clonedPix[pix]) + 4 * blue(clonedPix[pix1]) + blue(clonedPix[pix2])) / 11.0;
      pixels[pix] = color(r, g, b);
    }
  }
  updatePixels();
}

/* Gets a bounding box around a text string rendered on screen at (x, y).
The return value are an array of two vectors.
The first vector is the top-left corner of the box.
The second vector is the width and height of the box. */
PVector[] getBoundingBoxOfString(String s, float x, float y) {
  float ascent = textAscent();
  float descent = textDescent();
  float w = textWidth(s);
  PVector[] result = new PVector[2];
  result[0] = new PVector(x, y - ascent);
  result[1] = new PVector(w, ascent + descent);  
  return result;
}

/* Moves character ith (of the chimney text string). */
void animateChar(int i) {
  // if it's not the ith character's turn to appear, do nothing
  if (t < i * 1.0 / ChimneyText.length()) {
    return;
  }
  
  pushMatrix();
    translate(ChimneyTextX[i], ChimneyTextY[i]);
    scale(ChimneyTextS[i], ChimneyTextS[i]);
    textFont(ChimneyTextFont);
    textSize(48);
    fill(255);
    text(ChimneyText.charAt(i), 0, 0);
  popMatrix();
    
  ChimneyTextX[i] -= ChimneyTextDx[i];
  ChimneyTextY[i] -= ChimneyTextDy[i];
  ChimneyTextDx[i] += ChimneyTextSign[i] * 0.001;
  if (ChimneyTextDx[i] > 1 || ChimneyTextDx[i] < 0.5) {
    ChimneyTextSign[i] *= -1;
  }
  ChimneyTextDy[i] = 1 - ChimneyTextDx[i];
  ChimneyTextS[i] = min(ChimneyTextS[i] + 0.01, 1);
}