import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class DuongHoang_Week7_ImageType extends PApplet {

/*
This program consists of 3 consecutive "scenes".
Each scene is a static image with text that appears and disappears in some motion.
There are also transition motions between the scenes.
All the images used in this program are photos taken by me or given to me by a friend.
All the fonts used are freely available on the web. They are:
  - Cloud Home by Xerographer Fonts (www.dafont.com/cloudhome.font)
  - Lasting Love by Jonathan S. Harris (www.dafont.com/lasting-love.font)
  - McKloud Black by Jessica Slater (www.fontstock.net/270/mckloud-black.html)
  - Night Train 382 by Frederic Ric (www.dafont.com/night-train-382.font)  
*/

/* This is our global "time". It starts at 0 and increases from there.
The idea is that when T reaches 1, the second scene starts. When it reaches 2,
the third scene starts, and so on.
There's no way to speed up or slown down the animation,
but we can make T starts at 1 to begin from the second scene and so on. */
float T = 0.0f;
/* Each scenes also has its "local time" that it can use to synchronize the different motions in the scene. */
float t = 0;

public void setup() {
  smooth();
  size(1280, 800);

  /* train scene */
  TrainTextFont = loadFont("NightTrain382-60.vlw");
  ChimneyTextFont = loadFont("CloudHome-60.vlw");
  TrainImg = loadImage("train.png");
  CloudMixImg = loadImage("cloudmix.png");
  for (int i = 0; i < ChimneyText.length(); ++i) {
    ChimneyTextDx[i] = 1;
    ChimneyTextDy[i] = 0;
    ChimneyTextS[i] = 0;
    ChimneyTextX[i] = 1100;
    ChimneyTextY[i] = 300;
    ChimneyTextSign[i] = -1;
  }
  
  /* cloud scene */
  CloudImg = loadImage("cloud.png");
  CloudImg2 = loadImage("cloud2.png");
  FlowerImg = loadImage("flower7.png");
  CloudFont = loadFont("McKloudBlack-60.vlw");
  textFont(CloudFont);
  
  /* flower scene */
  for (int i = 0; i < FlowerImgs.length; ++i) {
    FlowerImgs[i] = loadImage("flower" + i + ".png");
  }
  FlowerFont = loadFont("LastingLove-48.vlw");
}

public void draw() {
  if (T >= 0 && T < 1) {
    drawTrainScene();    
  } else if (T >= 1 && T < 2) {
    drawCloudScene();
  } else if (T >= 2) {
    drawFlowerScene();
  }
  saveFrame();
}
/*
In this scene, an image of sky and clouds is shown.
Then a poem will scroll up in the left.
The poem seems to be about someone's love for his/her lost dearest person.
When the poem ends, the pixels will move somewhat randomly down and to the right.
This creates a flowing water effect.
Then random pixels will turn into that of the next scene's first frame, until all of them do.
*/

PImage CloudImg;
PImage CloudImg2;
PImage FlowerImg;

PFont CloudFont;

boolean CloudSceneInit = false;

float HeavenTextY = 830;
float HeavenTextYSpeed = -1;

/* This is the poem that will scroll up. */
String HeavenText =
  "If I could\n        visit heaven,\n\n\n\n" +
  "Even for a day,\n\n\n\n" +
  "Maybe for a moment,\n\n\n\n" +
  "The pain would\n        go away,\n\n\n\n" +
  "I'd put my arms\n        around you,\n\n\n\n" +
  "And whisper words\n        so true,\n\n\n\n" +
  "That living life\n        without you,\n\n\n\n" +
  "Is the hardest thing\n        to do.\n\n\n\n" +
  "No matter how\n    I spend my days,\n\n\n\n" +
  "No matter what\n        I do,\n\n\n\n" +
  "No morning dawns\n    or evening falls,\n\n\n\n" +
  "When I don't\n        think of you\n\n\n\n";

/* The probability that a cloud pixel becomes a flower pixel.
This will be increased as time goes. */
float FlowerPixProb = 0.001f;
  
public void drawCloudScene() {
  if (!CloudSceneInit) {
    initCloudScene();
    CloudSceneInit = true;
  }
  
  if (t < 2.5f) {
    /* display the scrolling poem */
    fill(255,254,253,255);    
    textFont(CloudFont);
    textSize(30);
    text(HeavenText, 70, HeavenTextY);
    HeavenTextY += HeavenTextYSpeed;
    loadPixels();
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < width; ++j) {      
        int pix = i * width + j;
        if (red(pixels[pix]) == 255 && green(pixels[pix]) == 254 && blue(pixels[pix]) == 253) {
          pixels[pix] = lerpColor(CloudImg.pixels[pix], CloudImg2.pixels[pix], 0);
        } else {
          // the background itself is a blend of two images
          pixels[pix] = lerpColor(CloudImg.pixels[pix], CloudImg2.pixels[pix], 0.5f);
        }
      }
    }
    updatePixels();
    t += 0.001f;
  } else {
    /* transit to the flower scene */
    loadPixels();
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < width; ++j) {
        int pix = i * width + j;
        int ii = i + PApplet.parseInt(random(0, 3));
        int jj = j + PApplet.parseInt(random(0, 3));
        if (ii >= height) ii = 0;
        if (jj >= width) jj = 0;
        int pixx = ii * width + jj;
        if (t > 5 && random(0, 1) < FlowerPixProb) {          
          pixels[pixx] = FlowerImg.pixels[ii * width + jj];
          FlowerPixProb += 0.0000001f;
        } else {
          pixels[pixx] = pixels[pix];
        }
      }
    }
    updatePixels();
    t += 0.002f;
    if (FlowerPixProb > 1) {
      /* end of this scene */
      T = 2;
      t = 0;            
    }
  }
}

public void initCloudScene() {  
  t = 0;
}
/*
In this scene, a blurred image of some flowers and leaves are shown.
The image then becomes clearer and clearer.
Then some text appears (in a smooth left-to-right way).
The text seems to carry some deep meaning. In fact it's just cryptic and meaningless.
When the text is all written down, the whole scene turns into white.
*/

PImage[] FlowerImgs = new PImage[8];
PFont FlowerFont;

String[] FlowerText = new String[] {
  "There is only one path to Heaven",
  "On Earth, we call it love",
  "'Cause love itself is just as innocent",
  "        as roses in May",
  "It is pure white",
  "        just like sin                                "
};

boolean FlowerSceneInit = false;

float FlowerImgIdx = FlowerImgs.length - 0.000001f;

int FlowerTextIdx = 0;

int FlowerTextAscent;
int FlowerTextDescent;
int FlowerTextWidth;
int FlowerTextHeight;
int[] FlowerTextMask;
float MaskT = 0;

int[] ScreenLastFrame;

public void drawFlowerScene() {  
  if (!FlowerSceneInit) {
    initFlowerScene();
    FlowerSceneInit = true;
  }
  
  if (FlowerImgIdx >= 1) { 
    /* lerp between the flower images */
    loadPixels();
    int img1 = PApplet.parseInt(FlowerImgIdx);
    int img2 = img1 - 1;        
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < width; ++j) {
        int pix = i * width + j;
        pixels[pix] = lerpColor(FlowerImgs[img1].pixels[pix], FlowerImgs[img2].pixels[pix], 1 - (FlowerImgIdx - img1));
      }
    }
    FlowerImgIdx -= 0.01f;
    updatePixels();
  } else if (FlowerTextIdx < FlowerText.length) {
    /* display the text using a masking technique */
    /* save last frame */
    loadPixels();
    arrayCopy(pixels, ScreenLastFrame);    
    /* draw the mask using a moving rectangle */  
    background(0);
    fill(230, 193, 176);
    int x = 120;
    int y = 400 + FlowerTextHeight * FlowerTextIdx;
    text(FlowerText[FlowerTextIdx], x, y);
    fill(0, 0, 0 ,255);  
    rect(map(MaskT, 0, 1, x, x + FlowerTextWidth + 10), y - FlowerTextAscent, FlowerTextWidth, FlowerTextHeight);
    MaskT += 0.005f;
    if (MaskT > 1) {
      MaskT = 0;
      ++FlowerTextIdx;
      if (FlowerTextIdx < FlowerText.length) {
        FlowerTextWidth = PApplet.parseInt(textWidth(FlowerText[FlowerTextIdx]) + 0.5f);
        FlowerTextMask = new int[FlowerTextWidth * (FlowerTextAscent + FlowerTextDescent)]; 
      }
    }    
    /* copy the mask from screen */
    loadPixels();
    for (int i = 0; i < FlowerTextHeight; ++i) {
      for (int j = 0; j < FlowerTextWidth; ++j) {
        FlowerTextMask[i * FlowerTextWidth + j] = pixels[(i + y - FlowerTextAscent) * width + (j + x)];
      }
    }    
    /* draw the last frame */
    arrayCopy(ScreenLastFrame, pixels);     
    /* draw the text */    
    for (int i = 0; i < FlowerTextHeight; ++i) {
      for (int j = 0; j < FlowerTextWidth; ++j) {
        int pix = i * FlowerTextWidth + j;
        if (red(FlowerTextMask[pix]) > 0 && green(FlowerTextMask[pix]) > 0) {
          pixels[(i + y) * width + (j + x)] = FlowerTextMask[i * FlowerTextWidth + j];
        }
      }
    }      
    updatePixels();
  } else {
    /* turn random pixels to white and end the scene */
    loadPixels();
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < width; ++j) {
        if (random(0, 1) < 0.5f) {
          continue;
        }
        int ii = i + PApplet.parseInt(random(-2, 2));
        int jj = j + PApplet.parseInt(random(-2, 2));
        int idx = i * width + j;
        if (red(pixels[idx]) == 255 && green(pixels[idx]) == 255 && blue(pixels[idx]) == 255) {
          continue;
        }        
        if (ii > 0 && jj > 0 && ii < height && jj < width) {        
          pixels[i * width + j] = pixels[ii * width + jj];
        } else {
          pixels[i * width + j] = color(255);
        }
      }
    }
    updatePixels();
  }
}

public void initFlowerScene() {
  textFont(FlowerFont);
  FlowerTextAscent = PApplet.parseInt(textAscent() + 0.5f);
  FlowerTextDescent = PApplet.parseInt(textDescent() + 0.5f);  
  FlowerTextHeight = FlowerTextAscent + FlowerTextDescent;
  FlowerTextWidth = PApplet.parseInt(textWidth(FlowerText[FlowerTextIdx]) + 0.5f);
  FlowerTextMask = new int[FlowerTextWidth * (FlowerTextAscent + FlowerTextDescent)];
  ScreenLastFrame = new int[width * height];  
}
/*
In this scene, an image of a train is displayed.
Then some text (a quote from the Inception movie) will be displayed.
Then some text will come out of the train's chimney, they will go to the left and up.
Then the camera will zoom-in, and the screen fades to the second scene.
*/

PFont TrainTextFont;
PFont ChimneyTextFont;
PImage TrainImg;
PImage CloudMixImg;

String[] TrainText = new String[] {
  "You're waiting for a train.",
  "A train that will take you far away.",
  "You know where you hope the train will take you,",
  "but you can't be sure.",
  "Yet it doesn't matter."
};

String ChimneyText = "Heaven7";
float[] ChimneyTextDx = new float[ChimneyText.length()];
float[] ChimneyTextDy = new float[ChimneyText.length()];
float[] ChimneyTextS = new float[ChimneyText.length()]; // scale
float[] ChimneyTextX = new float[ChimneyText.length()];
float[] ChimneyTextY = new float[ChimneyText.length()];
float[] ChimneyTextSign = new float[ChimneyText.length()];

// Whether the scene's variables have been initialized
boolean TrainSceneInit = false;

// Which string is being processed.
int TextIndex = 0;
float TextOpacity = 0;

float WholeSceneScale = 1;

PVector[] TextBox;

public void drawTrainScene() {
  if (!TrainSceneInit) {
    initTrainScene();
    TrainSceneInit = true;
  }
  
  if (TextIndex < TrainText.length) {
    /* show the quote */
    if (t < 0.5f) {    
      showText();
      TextOpacity += 0.2f;
      t += 0.003f;
    } else if (t >= 0.5f && t < 1) {    
      discardText();
      t += 0.003f;    
    } else if (t >= 1) {
      t = 0;
      ++TextIndex;
      TextOpacity = 0;
    }
    t += 0.001f;
  } else if (t < 1.4f) {
    /* show the chimney text */
    pushMatrix();
      if (t > 1) {
        scale(WholeSceneScale);
        WholeSceneScale += 0.005f;
      }
      image(TrainImg, 0, 0);
      for (int i = 0; i < ChimneyText.length(); ++i) {
        animateChar(i);
      }
    popMatrix();
    t += 0.001f;
  } else if (t >= 1.4f && t < 2.4f) {
    /* zoom-in and fade to the second scene */
    pushMatrix();
      scale(WholeSceneScale);
      WholeSceneScale += 0.005f;
      for (int i = 0; i < ChimneyText.length(); ++i) {
        animateChar(i);
      }
    popMatrix();
    loadPixels();
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < width; ++j) {
        int pix = i * width + j;
        pixels[pix] = lerpColor(pixels[pix], CloudMixImg.pixels[pix], t - 1.4f);
      }
    }
    updatePixels();
    t += 0.001f;
  } else {
    /* end of this scene */
    T = 1;
    t = 0;
  }
}

public void initTrainScene() {  
  t = 0;
  image(TrainImg, 0, 0);  
}

/* The Inception quote will be faded-in sentence by sentence (specified by TextIndex). */
public void showText() {
  if (TextIndex < TrainText.length) {
    fill(255, 255, 255, TextOpacity);
    textFont(TrainTextFont);
    textSize(60);
    text(TrainText[TextIndex], 100, 200);
    TextBox = getBoundingBoxOfString(TrainText[TextIndex], 100, 200);
  }
}

/* Discards text by blending each pixel with its left ones.
The text will appear to move to the right and then blends into the background. */
public void discardText() {
  loadPixels();
  int[] clonedPix = new int[pixels.length];
  arrayCopy(pixels, clonedPix);
  float x = TextBox[0].x;
  float y = TextBox[0].y;
  float w = TextBox[1].x;
  float h = TextBox[1].y;
  for (int i = PApplet.parseInt(y); i < y + h; ++i) {
    for (int j = PApplet.parseInt(x); j < x + w + 50; ++j) {           
      int j1 = j - 1;
      int j2 = j - 2;
      int pix = i * width + j;
      int pix1 = i * width + j1;
      int pix2 = i * width + j2;
      float r = (6 * red(clonedPix[pix]) + 4 * red(clonedPix[pix1]) + red(clonedPix[pix2])) / 11.0f;
      float g = (6 * green(clonedPix[pix]) + 4 * green(clonedPix[pix1]) + green(clonedPix[pix2])) / 11.0f;
      float b = (6 * blue(clonedPix[pix]) + 4 * blue(clonedPix[pix1]) + blue(clonedPix[pix2])) / 11.0f;
      pixels[pix] = color(r, g, b);
    }
  }
  updatePixels();
}

/* Gets a bounding box around a text string rendered on screen at (x, y).
The return value are an array of two vectors.
The first vector is the top-left corner of the box.
The second vector is the width and height of the box. */
public PVector[] getBoundingBoxOfString(String s, float x, float y) {
  float ascent = textAscent();
  float descent = textDescent();
  float w = textWidth(s);
  PVector[] result = new PVector[2];
  result[0] = new PVector(x, y - ascent);
  result[1] = new PVector(w, ascent + descent);  
  return result;
}

/* Moves character ith (of the chimney text string). */
public void animateChar(int i) {
  // if it's not the ith character's turn to appear, do nothing
  if (t < i * 1.0f / ChimneyText.length()) {
    return;
  }
  
  pushMatrix();
    translate(ChimneyTextX[i], ChimneyTextY[i]);
    scale(ChimneyTextS[i], ChimneyTextS[i]);
    textFont(ChimneyTextFont);
    textSize(48);
    fill(255);
    text(ChimneyText.charAt(i), 0, 0);
  popMatrix();
    
  ChimneyTextX[i] -= ChimneyTextDx[i];
  ChimneyTextY[i] -= ChimneyTextDy[i];
  ChimneyTextDx[i] += ChimneyTextSign[i] * 0.001f;
  if (ChimneyTextDx[i] > 1 || ChimneyTextDx[i] < 0.5f) {
    ChimneyTextSign[i] *= -1;
  }
  ChimneyTextDy[i] = 1 - ChimneyTextDx[i];
  ChimneyTextS[i] = min(ChimneyTextS[i] + 0.01f, 1);
}
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "DuongHoang_Week7_ImageType" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
